--[[ svn: $Id: InviteBlock.lua 39 2009-08-16 18:44:30Z crashmstr $

InviteBlock
    ...because they dont give up!

By crashmstr (wowzn@crashmstr.com)

Purpose:
This is an add-on to block incoming guild/party/trade/duel requests (except for friends)

Although I came up with the idea on my own, Ive borrowed a lot of the code from AntiSocial by Zariel / Kanne @ Trollbane.eu
Thank you!

(previously named ASSBlock)

--]]

InviteBlock = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

local options = {
	type='group',
	args = {
		enabled = {
			type = 'toggle', name = 'Enable', desc = 'Toggles enabled status',
			get = 'IsEnabled', set = 'ToggleEnabled',
		},
		friendsAllowed = {
			type = 'toggle', name = 'Allow Friends', desc = 'Toggles allowing friends',
			get = 'IsFriendsAllowed', set = 'ToggleFriendsAllowed',
		},
		partyBlock = {
			type = 'toggle', name = 'Block Party Invites', desc = 'Toggles party invite blocking',
			get = 'IsPartyBlocked', set = 'TogglePartyBlocked',
		},
		partyMessage = {
			type = 'text', name = 'Message for Blocked Party Invites',
			desc = 'The message to display in response to party invites',
			usage = "<your message here>",
			get = 'GetPartyMessage', set = 'SetPartyMessage',
		},
		tradeBlock = {
			type = 'toggle', name = 'Block Trade Invites', desc = 'Toggles trade blocking',
			get = 'IsTradeBlocked', set = 'ToggleTradeBlocked',
		},
		tradeMessage = {
			type = 'text', name = 'Message for Blocked Trades',
			desc = 'The message to display in response to trades',
			usage = "<your message here>",
			get = 'GetTradeMessage', set = 'SetTradeMessage',
		},
		duelBlock = {
			type = 'toggle', name = 'Block Duel Invites', desc = 'Toggles duel invite blocking',
			get = 'IsDuelBlocked', set = 'ToggleDuelBlocked',
		},
		duelMessage = {
			type = 'text', name = 'Message for Blocked Duel Invites',
			desc = 'The message to display in response to duel invites',
			usage = "<your message here>",
			get = 'GetDuelMessage', set = 'SetDuelMessage',
		},
		guildBlock = {
			type = 'toggle', name = 'Block Guild Invites', desc = 'Toggles guild invite blocking',
			get = 'IsGuildBlocked', set = 'ToggleGuildBlocked',
		},
		guildMessage = {
			type = 'text', name = 'Message for Blocked Guild Invites',
			desc = 'The message to display in response to guild invites',
			usage = "<your message here>",
			get = 'GetGuildMessage', set = 'SetGuildMessage',
		},
		reply = {
			type = 'toggle', name = 'Reply Message toggle', desc = 'Toggles the reply message',
			get = 'IsReply', set = 'ToggleReply',
		},
		followUp = {
			type = 'toggle', name = 'FollowUp Message toggle', desc = 'Toggles the followUp message',
			get = 'IsFollowUp', set = 'ToggleFollowUp',
		},
		followUpMessage = {
			type = 'text', name = 'FollowUp Message',
			desc = 'The message to display in addition to the reply message',
			usage = "<your message here>",
			get = 'GetFollowUpMessage', set = 'SetFollowUpMessage',
		},
		debug = {
			type = 'toggle', name = 'Debug window', desc = 'Toggles the debug window',
			get = 'IsDebug', set = 'ToggleDebug',
		},
	},
}

local stockMessage = "This is an automated reply to your request to dual/group/guild invite. I am using an add-on called InviteBlock to automatically decline since you are not on my friends list."
local defaults = {
	debug = true, enabled = true, reply = true, followUp = true,
	friendsAllowed = true, partyBlock = true,
	tradeBlock = false, duelBlock = true, guildBlock = true,
	message = {
		["GUILD_INVITE_REQUEST"] = stockMessage,
		["GUILD_REGISTRAR_SHOW"] = stockMessage,
		["PARTY_INVITE_REQUEST"] = stockMessage,
		["TRADE_SHOW"] = stockMessage,
		["DUEL_REQUESTED"] = stockMessage,
		["FOLLOW_UP"] = "If you think this is a mistake, please whisper back.",
	},
	blockLog = { },
}

InviteBlock:RegisterDB("InviteBlockDB", "InviteBlockDBPerChar")
InviteBlock:RegisterDefaults('profile', defaults)

function InviteBlock:IsEnabled() return self.db.profile.enabled; end
function InviteBlock:ToggleEnabled() self.db.profile.enabled = not self.db.profile.enabled; end
function InviteBlock:IsFriendsAllowed() return self.db.profile.friendsAllowed; end
function InviteBlock:ToggleFriendsAllowed() self.db.profile.friendsAllowed = not self.db.profile.friendsAllowed; end

function InviteBlock:IsPartyBlocked() return self.db.profile.partyBlock; end
function InviteBlock:TogglePartyBlocked() self.db.profile.partyBlock = not self.db.profile.partyBlock; end
function InviteBlock:GetPartyMessage() return self.db.profile.message["PARTY_INVITE_REQUEST"]; end
function InviteBlock:SetPartyMessage(msg) self.db.profile.message["PARTY_INVITE_REQUEST"] = msg; end

function InviteBlock:IsTradeBlocked() return self.db.profile.tradeBlock; end
function InviteBlock:ToggleTradeBlocked() self.db.profile.tradeBlock = not self.db.profile.tradeBlock; end
function InviteBlock:GetTradeMessage() return self.db.profile.message["TRADE_SHOW"]; end
function InviteBlock:SetTradeMessage(msg) self.db.profile.message["TRADE_SHOW"] = msg; end

function InviteBlock:IsDuelBlocked() return self.db.profile.duelBlock; end
function InviteBlock:ToggleDuelBlocked() self.db.profile.duelBlock = not self.db.profile.duelBlock; end
function InviteBlock:GetDuelMessage() return self.db.profile.message["DUEL_REQUESTED"]; end
function InviteBlock:SetDuelMessage(msg) self.db.profile.message["DUEL_REQUESTED"] = msg; end

function InviteBlock:IsGuildBlocked() return self.db.profile.guildBlock; end
function InviteBlock:ToggleGuildBlocked() self.db.profile.guildBlock = not self.db.profile.guildBlock; end
function InviteBlock:GetGuildMessage() return self.db.profile.message["GUILD_INVITE_REQUEST"]; end
function InviteBlock:SetGuildMessage(msg) self.db.profile.message["GUILD_INVITE_REQUEST"] = msg; end

function InviteBlock:IsReply() return self.db.profile.reply; end
function InviteBlock:ToggleReply() self.db.profile.reply = not self.db.profile.reply; end
function InviteBlock:IsFollowUp() return self.db.profile.followUp; end
function InviteBlock:ToggleFollowUp() self.db.profile.followUp = not self.db.profile.followUp; end
function InviteBlock:GetFollowUpMessage() return self.db.profile.message["FOLLOW_UP"]; end
function InviteBlock:SetFollowUpMessage(msg) self.db.profile.message["FOLLOW_UP"] = msg; end

function InviteBlock:IsDebug() return self.db.profile.debug; end
function InviteBlock:ToggleDebug() self.db.profile.debug = not self.db.profile.debug; end

function InviteBlock:OnInitialize()
    -- Called when the addon is loaded
end

function InviteBlock:OnEnable()
	-- Called when the addon is enabled
	self:RegisterEvent("GUILD_INVITE_REQUEST")
	self:RegisterEvent("GUILD_REGISTRAR_SHOW")
	self:RegisterEvent("PARTY_INVITE_REQUEST")
	self:RegisterEvent("TRADE_SHOW")
	self:RegisterEvent("TRADE_REQUEST")
	self:RegisterEvent("DUEL_REQUESTED")
	self:RegisterChatCommand("/assblock", "/assbl", options)
	self:Print("Anti-Social Software: Block invites (InviteBlock) ENABLED!")
end

function InviteBlock:OnDisable()
    -- Called when the addon is disabled
	self:Print("Anti-Social Software: Block invites (InviteBlock) DIS-ENABLED!")
end

function InviteBlock:IsFriend(name)
	local isFriend = false
	if name and self.db.profile.friendsAllowed then
		for i=1,GetNumFriends() do
			local name = GetFriendInfo(i)
			if name == arg1 then
				isFriend = true
				break
			end
		end
		if IsInGuild() then
			for i = 1,GetNumGuildMembers(includeOffline) do
				local name = GetGuildRosterInfo(i)
				if name == arg1 then
					isFriend = true
					break
				end
			end
		end
	end
	return isFriend
end

function InviteBlock:EnabledFor(type)
	local isEnabled = self.db.profile.enabled
	if isEnabled then
		if type == "TRADE_SHOW" then
			isEnabled = self.db.profile.tradeBlock
		elseif type == "DUEL_REQUESTED" then
			isEnabled = self.db.profile.duelBlock
		elseif type == "GUILD_INVITE_REQUEST" then
			isEnabled = self.db.profile.guildBlock
		elseif type == "PARTY_INVITE_REQUEST" then
			isEnabled = self.db.profile.partyBlock
		end
	end
	return isEnabled
end

function InviteBlock:SendReply(event, arg1)
	if self:IsReply() and arg1 then
		SendChatMessage(self.db.profile.message[event], "WHISPER", nil, arg1)
		if self:IsFollowUp() then
			SendChatMessage(self.db.profile.message["FOLLOW_UP"], "WHISPER", nil, arg1)
		end
	end
	self:LogBlock(event, arg1)
end

function InviteBlock:LogBlock(event, arg1)
	local index = 1 + table.getn(self.db.profile.blockLog)
	self.db.profile.blockLog[index] = {
		["date"] = date("%Y%m%d%H%M%S", time()),
		["event"] = event,
		["arg1"] = arg1,
		["player"] = UnitName('player'),
	}
end

function InviteBlock:GUILD_INVITE_REQUEST(arg1, arg2)
	self:Print("We were invited to join guild" .. (arg2 and arg2 or "(nil)") .. " by "
				  .. (arg1 and arg1 or "(nil)") .. "! wowz!")
	if self:EnabledFor(event) then
		if not self:IsFriend(arg1) then
			self:Print(arg1 .. " is not a friend, so send them a message and cancel the guild invite!")
			self:SendReply(event, arg1)
			HideUIPanel(StaticPopup1)
			DeclineGuild()
		end
	end
end

function InviteBlock:GUILD_REGISTRAR_SHOW(arg1, arg2)
	self:Print("We saw a guild registrar?!?"
				  .. (arg2 and arg2 or "nil") 
					  .. (arg1 and arg1 or "nil")
					  .. "! wowz!")
end

function InviteBlock:DUEL_REQUESTED(arg1)
	self:Print("We were invited to duel with " .. (arg1 and arg1 or "(nil)"))
	if self:EnabledFor(event) then
		if not self:IsFriend(arg1) then
			self:Print(arg1 .. " is not a friend, so send them a message and cancel the duel invite!")
			self:SendReply(event, arg1)
			HideUIPanel(StaticPopup1)
			CancelDuel()
		end
	end
end

function InviteBlock:PARTY_INVITE_REQUEST(arg1)
	self:Print("We were invited to party up by " .. (arg1 and arg1 or "(nil)"))
	if self:EnabledFor(event) then
		if not self:IsFriend(arg1) then
			self:Print(arg1 .. " is not a friend, so send them a message and cancel the party invite!")
			self:SendReply(event, arg1)
			HideUIPanel(StaticPopup1)
			DeclineGroup()
		end
	end
end

function InviteBlock:TRADE_SHOW(arg1)
	--this event does not get a parameter, so we cannot check the sender!?!?!?
	self:Print("We were invited to trade..." .. (arg1 and arg1 or "(nil)"))
	if self:EnabledFor(event) then
		--if not self:IsFriend(arg1) then
			--self:Print(arg1 .. " is not a friend, so send them a message and cancel the trade!")
			--self:SendReply(event, arg1)
			HideUIPanel(StaticPopup1)
			CancelTrade()
		--end
	end
end

function InviteBlock:TRADE_REQUEST(arg1)
	--this event does not fire
	self:Print("TRADE_REQUEST event..." .. (arg1 and arg1 or "(nil)"))
end
